/*
 * DemosceneMusic - OpenIdPage.java - Copyright © 2012 David Roden
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.pterodactylus.demoscenemusic.page;

import net.pterodactylus.demoscenemusic.core.Core;
import net.pterodactylus.util.template.Template;
import net.pterodactylus.util.template.TemplateContext;
import net.pterodactylus.util.template.TemplateContextFactory;
import net.pterodactylus.util.web.RedirectException;

import org.openid4java.association.AssociationException;
import org.openid4java.consumer.VerificationResult;
import org.openid4java.discovery.DiscoveryException;
import org.openid4java.discovery.DiscoveryInformation;
import org.openid4java.discovery.Identifier;
import org.openid4java.message.AuthSuccess;
import org.openid4java.message.MessageException;
import org.openid4java.message.ParameterList;
import org.openid4java.message.ax.AxMessage;
import org.openid4java.message.ax.FetchResponse;

/**
 * TODO
 *
 * @author <a href="mailto:bombe@pterodactylus.net">David ‘Bombe’ Roden</a>
 */
public class OpenIdPage extends BasePage {

	/**
	 * @param core
	 * @param templateContextFactory
	 * @param template
	 * @param pageName
	 */
	public OpenIdPage(Core core, TemplateContextFactory templateContextFactory, Template template) {
		super(core, templateContextFactory, template, "openid");
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void processTemplate(TemplateContext templateContext, ServletRequest request) throws RedirectException {
		super.processTemplate(templateContext, request);
		ParameterList parameterList = new ParameterList(request.getServletRequest().getParameterMap());
		DiscoveryInformation discoveryInformation = (DiscoveryInformation) request.getServletRequest().getAttribute("openid-discovery");
		templateContext.set("openid", discoveryInformation.getClaimedIdentifier());
		StringBuffer receivingURL = request.getServletRequest().getRequestURL();
		String queryString = request.getServletRequest().getQueryString();
		if (queryString != null && queryString.length() > 0) {
			receivingURL.append("?").append(queryString);
		}
		try {
			VerificationResult verification = getCore().getConsumerManager().verify(receivingURL.toString(), parameterList, discoveryInformation);
			Identifier verified = verification.getVerifiedId();
			if (verified != null) {
				System.out.println("verified.");
				templateContext.set("verified", true);
				templateContext.set("openid", verified.getIdentifier());
				AuthSuccess authSuccess = (AuthSuccess) verification.getAuthResponse();
				if (authSuccess.hasExtension(AxMessage.OPENID_NS_AX)) {
					FetchResponse fetchResponse = (FetchResponse) authSuccess.getExtension(AxMessage.OPENID_NS_AX);
					System.out.println(fetchResponse.getAttributes());
				}
			}
		} catch (MessageException me1) {
			me1.printStackTrace();
		} catch (DiscoveryException de1) {
			de1.printStackTrace();
		} catch (AssociationException ae1) {
			ae1.printStackTrace();
		}
	}

}
